/*********************************************************************
 *
 *                  PIC32 Boot Loader
 *
 *********************************************************************
 * FileName:        Bootloader.c
 * Dependencies:
 * Processor:       PIC32
 *
 * Compiler:        MPLAB C32
 *                  MPLAB IDE
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company?) for its PIC32 Microcontroller is intended
 * and supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PIC32 Microcontroller products.
 * The software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS? CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * $Id:  $
 * $Name: $
 *
 **********************************************************************/
#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "HardwareProfile.h"
#include "Bootloader.h"
#include "Uart.h"
//#include "FrameWork\Framework.h"
#include <stdlib.h>
#include <plib.h>


#if 1	//-------------- config bits
// Configuring the Device Configuration Registers
// 80Mhz Core/Periph, Pri Osc w/PLL, Write protect Boot Flash
#if defined(TRANSPORT_LAYER_USB) || defined(TRANSPORT_LAYER_USB_HOST)
    #pragma config UPLLEN   = ON        // USB PLL Enabled
    #pragma config UPLLIDIV = DIV_2     // USB PLL Input Divider = Divide by 2
#else
    #pragma config UPLLEN   = OFF       // USB PLL Enabled
#endif

#pragma config FUSBIDIO = OFF            // USB USID USBȊO̗prƂ(OFF)
#pragma config FVBUSONIO = OFF           // USB VBUS ON Selection (Controlled by USB Module)
#pragma config DEBUG    = OFF           // Background Debugger disabled

// NbNݒ
#pragma config POSCMOD = HS             // OtIV[^
#pragma config FNOSC = PRIPLL           // OtPLLU
#pragma config FSOSCEN = OFF            // Secondary Oscillator Enable (Disabled)
#pragma config IESO = OFF               // Internal/External Switch Over (Disabled)
#pragma config OSCIOFNC = OFF           // CLKO Output Signal Active on the OSCO Pin (Disabled)
#pragma config FPBDIV = DIV_1           // Peripheral Bus Clock: Divide by 1
#pragma config FPLLMUL = MUL_20         // PLL Multiplier: Multiply by 20
#pragma config FPLLIDIV = DIV_5         // 20 / 5 = 4, 4 x 20 = 80
#if defined(__PIC32MX1XX_2XX__)
    // For PIC32MX1xx, PIC32MX2xx devices the output divisor is set to 2 to produce max 40MHz clock.
    #pragma config FPLLODIV = DIV_2     // PLL Output Divider: 80 / 2 = 40
#elif defined (__PIC32MX3XX_7XX__)
    //For PIC32MX3xx, PIC32MX4xx, PIC32MX5xx, PIC32MX6xx and PIC32MX7xx devices, 
    //the output divisor is set to 1 to produce max 80MHz clock.
    #pragma config FPLLODIV = DIV_1         // PLL Output Divider: Divide by 1
#endif

#pragma config FWDTEN = OFF             // WD timer: OFF
#pragma config BWP = OFF                // Boot write protect: OFF
#pragma config JTAGEN = OFF             // ftHgIȂ̂TxxsgꍇK{ŃItɂ邱ƁB)

#if defined(__PIC32MX1XX_2XX__)
    // For PIC32MX1xx, PIC32MX2xx devices there are jumpers on PIM to choose from PGx1/PGx2.
    #pragma config ICESEL = ICS_PGx3    // ICE pins configured on PGx1 (PGx2 is multiplexed with USB D+ and D- pins).
#elif defined(__PIC32MX3XX_7XX__)
    // For PIC32MX3xx, PIC32MX4xx, PIC32MX5xx, PIC32MX6xx and PIC32MX7xx 
    // devices the ICE connection is on PGx2. .
    #pragma config ICESEL = ICS_PGx2    // ICE pins configured on PGx2, Boot write protect OFF.
#endif




#if defined(TRANSPORT_LAYER_ETH)
	#pragma config FMIIEN = OFF, FETHIO = OFF	// external PHY in RMII/alternate configuration. Applicable for devices with internal MAC only.
#endif

#define SWITCH_PRESSED 0
#define ReadSwitchStatus()  1
#endif

//#define InitLED()
//#define BlinkLED()

#define BlinkLED() (LATBbits.LATB7 = (PORTBbits.RB7 != 1))
#define InitLED() do{TRISBbits.TRISB7 = 0; LATBbits.LATB7= 0;}while(0)
#define ErrorReset() {  PutChar('Z');   PutChar('0');     Reset(); }
#define led_on()	(LATBbits.LATB7 = 0)
#define led_off()	(LATBbits.LATB7 = 1)

// BootloadervOĂۂɁAo̓|[gIɂȂȂ悤ɂĂ
// PORT B8, B9  1 ̎FETItɂȂ
#define OUTPUTPORTSMSK	(_TRISB_TRISB8_MASK | _TRISB_TRISB9_MASK)
#define OUTPUTPORTOFF()	(LATBSET = _LATB_LATB9_MASK | _LATB_LATB8_MASK)

#define PairTriggerOutput()	(TRISBbits.TRISB15 = 0)
#define PairTriggerInput()	(TRISBbits.TRISB15 = 1)
#define PairTriggerOn()	(LATBbits.LATB15 = 1)
#define PairTriggerOff()	(LATBbits.LATB15 = 0)

void ConvertAsciiToHex(UINT8* asciiRec, UINT8* hexRec);
void EraseFlash(void);
void WriteHexRecord2Flash(UINT8* HexRecord);

UINT8 asciiRec[200];
UINT8 hexRec[100];

typedef struct 
{
	UINT8 RecDataLen;
	DWORD_VAL Address;
	UINT8 RecType;
	UINT8* Data;
	UINT8 CheckSum;	
	DWORD_VAL ExtSegAddress;
	DWORD_VAL ExtLinAddress;
}T_HEX_RECORD;	

//-------------- u[g[_Ɩ{̂ŏ񋤗Lׂ̈ɕێtbV̈̃f[^\
typedef struct _FLUSHWK
{
	int		IsFlashEnable;
	//---->	Add Here
	//<----
	UINT8	filler[0x400 - sizeof(int)];	// S1KoCgł
}FLUSHWK, *PFLUSHWK;

/******************************************************************************
Macros used in this file
*******************************************************************************/
#define SWITCH_PRESSED 0
#define DEV_CONFIG_REG_BASE_ADDRESS 	0x9FC00BF0  // DevCoñAhXA@ˑKύX
#define DEV_CONFIG_REG_END_ADDRESS   	0x9FC00BFF

//-----------------[U[f[^̈ƂāA0x2000oCg0x9D004000mۂĂ̂Ƃ
#define UDATA_FLASH_ADDRESS         0x9D004000      // ̃vOŎgpłtbV̈̐擪
#define UDATA_WORK_ADDRESS			(UDATA_FLASH_ADDRESS + 0x0400)
#define UDATA_WORK_SIZE				(sizeof(FLUSHWK))
#define TBL01_FLASH_ADDRESS         0x9D004800      // 
    
#define UDATA_FLASH_END_ADDRESS   (0x9D004000 + 0x2000)



#ifdef __cplusplus
extern "C" {
#endif
BOOL CheckTrigger();
void JumpToApp(void);
BOOL ValidAppPresent(void);
#ifdef __cplusplus
}
#endif

void Delay10us(DWORD dwCount)
{
	volatile DWORD _dcnt;

	_dcnt = dwCount*((DWORD)(0.00001/(1.0/GetInstructionClock())/10));
	while(_dcnt--)
	{
		#if defined(__C32__)
			Nop();
			Nop();
			Nop();
		#endif
	}
}

void DelayMs(WORD ms)
{
    unsigned char i;
    while(ms--)
    {
        i=4;
        while(i--)
        {
            Delay10us(25);
        }
    }
}

//--------------------------------------- WkFlush̍XV
void UpdateWkFlush(int IsFlashEnable) 
{
	FLUSHWK	buff;
	int	 *pflush = (int *)UDATA_WORK_ADDRESS,
			*pwk = (int *)&buff;
	int		i;
// ݂̏ޔĂ	
	for(i = 0; i < (sizeof(FLUSHWK) / sizeof(int)); i++)
		pwk[i] = pflush[i];
	buff.IsFlashEnable = IsFlashEnable;
//@Yӏ
	NVMemErasePage(pflush);		// 1024PʂŏȂ̂ŁA1񂾂ł̍\̕ɂȂ
//	[hPʂŏ݂Ă
	pflush = (int *)PA_TO_KVA0(UDATA_WORK_ADDRESS);
	for(i = 0; i < (sizeof(FLUSHWK) / sizeof(int)); i++)
		NVMemWriteWord(&pflush[i], pwk[i]);	
}

INT main(void)
{
	UINT pbClk; 
	unsigned uadresult;
	PFLUSHWK	pwkflush;

	// Setup configuration
	pbClk = SYSTEMConfig(SYS_FREQ, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

	// BootloadervOĂۂɁAo̓|[gIɂȂȂ悤ɖIɃNAĂ
	TRISBCLR = OUTPUTPORTSMSK;
	OUTPUTPORTOFF();

	InitLED();
	led_off();

	// n[hEFAgKA܂͋ʃtbV̈āAvOXV[hǂ肷
	pwkflush = (PFLUSHWK)UDATA_WORK_ADDRESS;
	if(!CheckTrigger() && pwkflush->IsFlashEnable != 1)
	{
		PairTriggerOutput();		// B15yA\ǂ`邽߂ɎgpB
		PairTriggerOff();		// Ƃ肠sɂĂ
		
		// XbgZT̂̕ADϊsAdȏ゠ONƂ
		ANSELA = 0x03;				// A0, A1̓AiO for sensor
		ANSELB = 0x0f;				// B0 - B3 AiOAfor sensor
		TRISBbits.TRISB2 = 1;
		// ftHgANSEL0xffffȂ̂ŁAAiO͂ƂȂ
		AD1CON1 = 0;				// Ƃ肠NAIt
		AD1CON2 = 0;				// vAobt@gpȂ̂0
		// Ȃ̂ŁASŃTvO
		AD1CON3bits.SAMC = 1;		// Ȃ̂ňӖȂ
		AD1CON3bits.ADCS = 1;		// 4ɂĂ݂  
		AD1CHS = 0x00040000;		// AN4CH0
		AD1CSSL = 0x10;				// AN4TvO

		AD1CON1bits.ON = 1;			// ADϊW[ON
		AD1CON1bits.SAMP = 1;		// ϊJn

		DelayMs(1);
		AD1CON1bits.SAMP = 0;		// ϊI
		while(!AD1CON1bits.DONE);
		uadresult = ADC1BUF0;
		if(ADC1BUF0 > 512)	// ȏ゠΃IƂ(ANZJxȏ) 
			PairTriggerOn();
		AD1CON1bits.ON = 0;			// ADϊW[OFF
	}
	else
	{
		char	IsFlushOkay = 0;
		TRANS_LAYER_Init(pbClk);
		DelayMs(1);

		// s؂̃f[^ŁAt@C̍Ō0Ń^[~l[gĂ̂ĂďI
		UINT8   byte1 = 0;

		while(1)
		{
			if(!GetChar(&byte1))
			{
				led_off();
				PutChar(_BEL);               // x
				TRANS_LAYER_Close();
				Reset();
			}
			if(byte1 == '6')
			{
				DelayMs(2);	
				PutChar('4');               // XVmF
			}
			else if(byte1 == '5')
			{
				int     i;
				PutChar('5');               // f[^擾v
				led_on();
				for(i = 0; i < 200; i++)
				{
					if(!GetChar(&byte1))
					{
						led_off();
						PutChar(_BEL);               // x
						TRANS_LAYER_Close();
						Reset();
					}
					asciiRec[i] = byte1;
					if (byte1 == 0x0A || byte1 == 0)
						 break;
				}
				ConvertAsciiToHex(&asciiRec[1], hexRec);
				WriteHexRecord2Flash(hexRec);
				if(byte1)
					PutChar('4');			// p
				else
				{
					led_off();
					IsFlushOkay = 1;
					PutChar('9');			// I
					break;
				}
//				BlinkLED();	
				led_off();
			}
			else if(byte1 == '9')
			{
										// I
				break;
			}
			else if(byte1 == 'u')
			{
				EraseFlash();
				PutChar('4');			// XVmF[v
			}
			else if(byte1 == _DC1)	{	// mF
				BlinkLED();
				PutChar(_ACK);
			}
			else
			{
				PutChar(_BEL);			// xԐM
			}
		}
		DelayMs(10);	
	    TRANS_LAYER_Close();   	
		if(IsFlushOkay)
			UpdateWkFlush(0);
	}

	// ALȃAvȂꍇAXVw߂܂őҋ@
	if(!ValidAppPresent())
	{
		UINT8   byte1 = 0;
		TRANS_LAYER_Init(pbClk);
		while(1)
		{
			if(GetChar(&byte1))
			{
				if(byte1 == 'u')
				{
					PutChar(_ACK);
					DelayMs(10);
					TRANS_LAYER_Close();
					UpdateWkFlush(1);
					DelayMs(10);
					Reset();
					break;
				}
				else if(byte1 == _DC1)
				{
					BlinkLED();
					PutChar(_ACK);
				}
				else
				{
					PutChar(_NAK);
				}
			}
		}
	    TRANS_LAYER_Close();   	
	}
	
	JumpToApp();
	
	return 0;
}			


/********************************************************************
* Function: 	CheckTrigger()
*
* Precondition: 
*
* Input: 		None.
*
* Output:		TRUE: If triggered
				FALSE: No trigger
*
* Side Effects:	None.
*
* Overview: 	Checks if there is a trigger to enter 
				firmware upgrade mode.
*
*			
* Note:		 	None.
********************************************************************/
BOOL  CheckTrigger()
{
	return FALSE;		
}	


/********************************************************************
* Function: 	JumpToApp()
*
* Precondition: 
*
* Input: 		None.
*
* Output:		
*
* Side Effects:	No return from here.
*
* Overview: 	Jumps to application.
*
*			
* Note:		 	None.
********************************************************************/
void JumpToApp(void)
{	
	void (*fptr)(void);
	fptr = (void (*)(void))USER_APP_RESET_ADDRESS;
	fptr();
}	


/********************************************************************
* Function: 	ValidAppPresent()
*
* Precondition: 
*
* Input: 		None.
*
* Output:		TRUE: If application is valid.
*
* Side Effects:	None.
*
* Overview: 	Logic: Check application vector has 
				some value other than "0xFFFFFFFF"
*
*			
* Note:		 	None.
********************************************************************/
BOOL ValidAppPresent(void)
{
	DWORD *AppPtr;
	AppPtr = (DWORD *)USER_APP_RESET_ADDRESS;
	if(*AppPtr == 0xFFFFFFFF)
	{
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}			
/********************************************************************
* Function: 	ConvertAsciiToHex()
*
* Precondition: 
*
* Input: 		Ascii buffer and hex buffer.
*
* Output:		
*
* Side Effects:	No return from here.
*
* Overview: 	Converts ASCII to Hex.
*
*			
* Note:		 	None.
********************************************************************/
void ConvertAsciiToHex(UINT8* asciiRec, UINT8* hexRec)
{
	UINT8 i = 0;
	UINT8 k = 0;
	UINT8 hex;
	
	
	while((asciiRec[i] >= 0x30) && (asciiRec[i] <= 0x66))
	{
		// Check if the ascci values are in alpha numeric range.
		
		if(asciiRec[i] < 0x3A)
		{
			// Numerical reperesentation in ASCII found.
			hex = asciiRec[i] & 0x0F;
		}
		else
		{
			// Alphabetical value.
			hex = 0x09 + (asciiRec[i] & 0x0F);						
		}
	
		// Following logic converts 2 bytes of ASCII to 1 byte of hex.
		k = i%2;
		
		if(k)
		{
			hexRec[i/2] |= hex;
			
		}
		else
		{
			hexRec[i/2] = (hex << 4) & 0xF0;
		}	
		i++;		
	}		
	
}

// Do not change this
#define FLASH_PAGE_SIZE 0x0400
/********************************************************************
* Function: 	EraseFlash()
*
* Precondition: 
*
* Input: 		None.
*
* Output:		
*
* Side Effects:	No return from here.
*
* Overview: 	Erases Flash (Block Erase).
*
*			
* Note:		 	None.
********************************************************************/
void EraseFlash(void)
{
	void * pFlash;
    UINT result;
    INT i;

    pFlash = (void*)APP_FLASH_BASE_ADDRESS;									
    for( i = 0; i < ((APP_FLASH_END_ADDRESS - APP_FLASH_BASE_ADDRESS + 1)/FLASH_PAGE_SIZE); i++ )
    {
	     result = NVMemErasePage( pFlash + (i*FLASH_PAGE_SIZE) );
        // Assert on NV error. This must be caught during debug phase.

        if(result != 0)
        {
           // We have a problem. This must be caught during the debug phase.
            while(1);
        } 
        // Blink LED to indicate erase is in progress.
        BlinkLED();
    }			           	     
}



/********************************************************************
* Function: 	WriteHexRecord2Flash()
*
* Precondition: 
*
* Input: 		None.
*
* Output:		
*
* Side Effects:	No return from here.
*
* Overview: 	Writes Hex Records to Flash.
*
*			
* Note:		 	None.
********************************************************************/
void WriteHexRecord2Flash(UINT8* HexRecord)
{
#define DATA_RECORD 		0
#define END_OF_FILE_RECORD 	1
#define EXT_SEG_ADRS_RECORD 2
#define EXT_LIN_ADRS_RECORD 4
	static T_HEX_RECORD HexRecordSt;
	UINT8 Checksum = 0;
	UINT8 i;
	UINT WrData;
	UINT RdData;
	void* ProgAddress;
	UINT result;
		
	HexRecordSt.RecDataLen = HexRecord[0];
	HexRecordSt.RecType = HexRecord[3];	
	HexRecordSt.Data = &HexRecord[4];	
	
	// Hex Record checksum check.
	for(i = 0; i < HexRecordSt.RecDataLen + 5; i++)
	{
		Checksum += HexRecord[i];
	}	
	
    if(Checksum != 0)
    {
	    //Error. Hex record Checksum mismatch.
	    //Indicate Error by switching ON all LEDs.
	    ErrorReset();
	    // Do not proceed further.
	    while(1);
	} 
	else
	{
		// Hex record checksum OK.
		switch(HexRecordSt.RecType)
		{
			case DATA_RECORD:  //Record Type 00, data record.
				HexRecordSt.Address.byte.MB = 0;
					HexRecordSt.Address.byte.UB = 0;
					HexRecordSt.Address.byte.HB = HexRecord[1];
					HexRecordSt.Address.byte.LB = HexRecord[2];
					
					// Derive the address.
					HexRecordSt.Address.Val = HexRecordSt.Address.Val + HexRecordSt.ExtLinAddress.Val + HexRecordSt.ExtSegAddress.Val;
							
					while(HexRecordSt.RecDataLen) // Loop till all bytes are done.
					{
											
						// Convert the Physical address to Virtual address. 
						ProgAddress = (void *)PA_TO_KVA0(HexRecordSt.Address.Val);
						
						// Make sure we are not writing boot area and device configuration bits.
						if(((ProgAddress >= (void *)APP_FLASH_BASE_ADDRESS) && (ProgAddress <= (void *)APP_FLASH_END_ADDRESS))
						   && ((ProgAddress < (void*)DEV_CONFIG_REG_BASE_ADDRESS) || (ProgAddress > (void*)DEV_CONFIG_REG_END_ADDRESS)))
						{
							if(HexRecordSt.RecDataLen < 4)
							{
								
								// Sometimes record data length will not be in multiples of 4. Appending 0xFF will make sure that..
								// we don't write junk data in such cases.
								WrData = 0xFFFFFFFF;
								memcpy(&WrData, HexRecordSt.Data, HexRecordSt.RecDataLen);	
							}
							else
							{	
								memcpy(&WrData, HexRecordSt.Data, 4);
							}		
							// Write the data into flash.	
							result = NVMemWriteWord(ProgAddress, WrData);	
							// Assert on error. This must be caught during debug phase.		
							if(result != 0)
							{
    							while(1);
    						}									
						}	
						
						// Increment the address.
						HexRecordSt.Address.Val += 4;
						// Increment the data pointer.
						HexRecordSt.Data += 4;
						// Decrement data len.
						if(HexRecordSt.RecDataLen > 3)
						{
							HexRecordSt.RecDataLen -= 4;
						}	
						else
						{
							HexRecordSt.RecDataLen = 0;
						}	
					}
					break;
			
			case EXT_SEG_ADRS_RECORD:  // Record Type 02, defines 4th to 19th bits of the data address.
			    HexRecordSt.ExtSegAddress.byte.MB = 0;
				HexRecordSt.ExtSegAddress.byte.UB = HexRecordSt.Data[0];
				HexRecordSt.ExtSegAddress.byte.HB = HexRecordSt.Data[1];
				HexRecordSt.ExtSegAddress.byte.LB = 0;
				// Reset linear address.
				HexRecordSt.ExtLinAddress.Val = 0;
				break;
				
			case EXT_LIN_ADRS_RECORD:   // Record Type 04, defines 16th to 31st bits of the data address. 
				HexRecordSt.ExtLinAddress.byte.MB = HexRecordSt.Data[0];
				HexRecordSt.ExtLinAddress.byte.UB = HexRecordSt.Data[1];
				HexRecordSt.ExtLinAddress.byte.HB = 0;
				HexRecordSt.ExtLinAddress.byte.LB = 0;
				// Reset segment address.
				HexRecordSt.ExtSegAddress.Val = 0;
				break;
				
			case END_OF_FILE_RECORD:  //Record Type 01, defines the end of file record.
				HexRecordSt.ExtSegAddress.Val = 0;
				HexRecordSt.ExtLinAddress.Val = 0;
				// Disable any interrupts here before jumping to the application.
				//!!!JumpToApp();
				break;
				
			default: 
				HexRecordSt.ExtSegAddress.Val = 0;
				HexRecordSt.ExtLinAddress.Val = 0;
				break;
		}		
	}	
		
}	

/*********************End of File************************************/


