/*********************************************************************
 *
 *                  PIC32 Boot Loader
 *
 *********************************************************************
 * FileName:        Uart.c
 * Dependencies:
 * Processor:       PIC32
 *
 * Compiler:        MPLAB C32
 *                  MPLAB IDE
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company?) for its PIC32 Microcontroller is intended
 * and supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PIC32 Microcontroller products.
 * The software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS? CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * $Id:  $
 * $Name: $
 *
 **********************************************************************/
#include <p32xxxx.h>
#include <stdlib.h>
#include <plib.h>
#include "Uart.h"
#include "HardwareProfile.h"
#include "BootLoader.h"
#include "Framework\Framework.h"


static UINT8 TxBuff[255];
#ifdef __cplusplus
extern "C" {
#endif
BOOL GetChar(UINT8 *byte);
void PutChar(UINT8 txChar);
#ifdef __cplusplus
}
#endif


/********************************************************************
* Function: 	UartInit()
*
* Precondition: 
*
* Input: 		PB Clock
*
* Output:		None.
*
* Side Effects:	None.
*
* Overview:     Initializes UART.
*
*			
* Note:		 	None.
********************************************************************/	
void UartInit(UINT pbClk)
{
//	#if   (((__PIC32_F EATURE_SET__ >= 100) && (__PIC32_FEATURE_SET__ <= 299)))
#ifdef COMSPI
        SDI1R = 3;                  // B13SDO1
        RPB13R = 3;                 // B11SDI1
        TRISBbits.TRISB13 = 0;      //  SDO1, OUTPUT
        TRISBbits.TRISB11 = 1;      // SDI1 , INPUT
        TRISBbits.TRISB14 = 0;      // SCK1, Output
		CNPUBbits.CNPUB11 = 1;		// SDI1. pllup
        // SPI, 20炢2Mbps
        OpenSPI1(MASTER_ENABLE_ON|SEC_PRESCAL_5_1|PRI_PRESCAL_64_1, SPI_ENABLE);
#else
		U2RXR = 3;					// U2RX B11
        RPB14R = 2;                 // B13U2TX
        TRISBbits.TRISB11 = 1;      // U2RX , INPUT
        TRISBbits.TRISB14 = 0;      // U2TX, Output
//		CNPUBbits.CNPUB11 = 1;		// U2RX. pllup
   //     TRISBbits.TRISB13 = 0;      // T

		//PPSInput(2,U2RX, RPB1);
		//PPSOutput(4, RPB0, U2TX);
		// Open UART2 with Receive and Transmitter enable.
		UxBRG = (pbClk/16/DEFAULT_BAUDRATE-1); // calculate actual BAUD generate value.
		UxMODE = UART_EN;
		UxSTA = (UART_RX_ENABLE | UART_TX_ENABLE);	
	
	#endif
}	


/********************************************************************
* Function: 	UartClose()
*
* Precondition: 
*
* Input: 		None
*
* Output:		None.
*
* Side Effects:	None.
*
* Overview:     Closes UART connection.
*
*			
* Note:		 	None.
********************************************************************/	
void UartClose(void)
{
#ifdef COMSPI
	CloseSPI1();
#else
//	char dummy;
//	if(UxSTAbits.URXDA)
//		dummy = UxRXREG;		        // get data from UART RX FIFO
	CloseUART();
	UARTClearAllErrors();
#endif
}	


/********************************************************************
* Function: 	UartTasks()
*
* Precondition: 
*
* Input: 		None
*
* Output:		None
*
* Side Effects:	None.
*
* Overview:     Runs some UART routines like receive bytes and transmit bytes.
*
*			
* Note:		 	None.
********************************************************************/
#if 0
void UartTask(void)
{
	UINT8 TxLen;
	UINT8 Rx;
	UINT8 *ptr;
	// Check any character is received.
	if(GetChar(&Rx))
	{
		// Pass the bytes to frame work.
		FRAMEWORK_BuildRxFrame(&Rx, 1);
	}	
	
	ptr = TxBuff;
	// Get transmit frame from frame work.
	TxLen = FRAMEWORK_GetTransmitFrame(ptr);
	
	if(TxLen)
	{
		// There is something to transmit.
	   	while(TxLen--)
	   	{
		   PutChar(*(ptr++));
		} 
	} 
	
	
}

#endif
/********************************************************************
* Function: 	GetChar()
*
* Precondition: 
*
* Input: 		None
*
* Output:		True: If there is some data.
*
* Side Effects:	None.
*
* Overview:     Gets the data from UART RX FIFO.
*
*			
* Note:		 	None.
********************************************************************/
BOOL GetChar(UINT8 *byte)
{
	BYTE dummy;
//   int     nTry = GetInstructionClock() * ERROR_TIMEOUT;
#ifdef COMSPI
 
 //   dummy = SPI1BUF;                // G[œǂݏoĂȂꍇ̑΍Ń_~[ǂݍ
 //   SPI1BUF = 0xff;					// ǂݏoOɃ_~[݂ŃNbN
 //   while(!SPI1STATbits.SPIRBF && nTry)
 //       nTry--;    // 

 //   if(!SPI1STATbits.SPIRBF)
 //       return FALSE;
 //   *byte = SPI1BUF;		        // get data from UART RX FIFO
	putcSPI1(0xff);					// ǂݏoOɃ_~[݂ŃNbN
	*byte = getcSPI1();
	return TRUE;
#else

	if(UxSTA & _U1ASTA_OERR_MASK)
		UxSTAbits.OERR = 0;			// I[o[t[͂ŃNA邵Ȃ
	
	if(UxSTA & (_U1ASTA_FERR_MASK | _U1ASTA_PERR_MASK))	// t[G[ƃpeBG[͂
		dummy = UxRXREG; 			// dummy read to clear FERR/PERR

	//	while(!UxSTAbits.URXDA && --nTry > 0);
	while(!UxSTAbits.URXDA);
	if(UxSTAbits.URXDA)
	{
		*byte = UxRXREG;		        // get data from UART RX FIFO
		return TRUE;
	}
	
	return FALSE;

#endif

}


/********************************************************************
* Function: 	PutChar()
*
* Precondition: 
*
* Input: 		None
*
* Output:		None
*
* Side Effects:	None.
*
* Overview:     Puts the data into UART tx reg for transmission.
*
*			
* Note:		 	None.
********************************************************************/
void PutChar(UINT8 txChar)
{
#ifdef COMSPI
	BYTE dummy;
//    while(!SPI1STATbits.SPITBE); // wait for TX buffer to be empty
//    SPI1BUF = txChar;
//    while(!SPI1STATbits.SPIRBF); // Mҋ@
    dummy = SPI1BUF;            // _~[ǂݏo
	putcSPI1(txChar);					
	dummy = getcSPI1();				// _~[ǂݏo
#else
	while(UxSTAbits.UTXBF); // wait for TX buffer to be empty
    UxTXREG = txChar;
#endif

    
}

/***************************************End Of File*************************************/
